@@ -162,7 +162,7 @@ Type RCM(objective_function<Type> *obj) {
 
     if(y <= ageM) E0_SR += E0(y);
   }
-  E0_SR /= Type(ageM);
+  E0_SR /= Type(ageM + 1);
   Type EPR0_SR = E0_SR/R0;
 
   Type CR_SR, Brec;
